<?php


namespace FiloBlu\Referrals\Helper;

use Magento\Framework\App\Helper\AbstractHelper;


class Core extends AbstractHelper
{

    // Do not change this
    const INVITATION_COUPON_PREFIX = 'INV-';

    const COUPON_FIELD_INVITATION_TYPE = 'filoblu_referrals/general/referral_invitation_type';
    const COUPON_FIELD_NEWSLETTER_SUBSCRIPTION_REDIRECT = 'filoblu_referrals/general/newsletter_subscription_redirect';
    const COUPON_FIELD_NAME_FOR_WHO_SEND_THE_INVITE = 'coupon_for_who_sent_the_invite';
    const COUPON_FIELD_NAME_FOR_WHO_GET_INVITED = 'coupon_for_who_got_invited';

    private $_customerRepositoryInterface;
    protected $ruleFactory;
    protected $massGeneratorFactory;
    protected $couponFactory;
    protected $coupon;
    protected $ruleResource;
    protected $invitationCollectionFactory;

    protected $_resource;
    protected $connection;

    /**
     * @param \Magento\Framework\App\Helper\Context $context
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Customer\Api\CustomerRepositoryInterface $customerRepositoryInterface,
        \Magento\SalesRule\Model\ResourceModel\Rule $rulesResource,
        //\Magento\Framework\Stdlib\DateTime\DateTime $dateTime,
        \Magento\SalesRule\Model\Coupon\MassgeneratorFactory $massgeneratorFactory,
        \Magento\SalesRule\Model\CouponFactory $couponFactory,
        \Magento\SalesRule\Model\ResourceModel\Coupon $coupon,
        \Magento\SalesRule\Model\RuleFactory $ruleFactory,
        \Magento\Invitation\Model\ResourceModel\Invitation\CollectionFactory $invitationCollectionFactory,
        \Magento\Framework\App\ResourceConnection $resource
    ) {
        parent::__construct($context);
        $this->_customerRepositoryInterface = $customerRepositoryInterface;
        $this->ruleResource = $rulesResource;
        //$this->datetime = $dateTime;
        $this->massGeneratorFactory = $massgeneratorFactory;
        $this->couponFactory = $couponFactory;
        $this->coupon = $coupon;
        $this->ruleFactory = $ruleFactory;
        $this->invitationCollectionFactory = $invitationCollectionFactory;
        $this->_resource = $resource;
    }

    /**
     * @return bool
     */
    public function isEnabled()
    {
        return $this->scopeConfig->getValue('filoblu_referrals/general/enabled');
    }

    protected function getConnection()
    {
        if (!$this->connection) {
            $this->connection = $this->_resource->getConnection('core_read');
        }
        return $this->connection;
    }

    /**
     * Simple function to write logs.
     * Zero hassle. Works always.
     */
    public function writeLog($message, $log_name = 'referrals.log'){
        $writer = new \Zend\Log\Writer\Stream(BP . "/var/log/{$log_name}");
        $logger = new \Zend\Log\Logger();
        $logger->addWriter($writer);
        $logger->info($message);

    }

    public function generateCoupon($couponCodeId, $expireDate = false)
    {
        if ($couponCodeId) {
            $rule = $this->ruleFactory->create();
            $this->ruleResource->load($rule, $couponCodeId);

            $generator = $this->massGeneratorFactory->create();
            $generator->setFormat(
                \Magento\SalesRule\Helper\Coupon::COUPON_FORMAT_ALPHANUMERIC
            );
            $generator->setRuleId($couponCodeId);
            $generator->setUsesPerCoupon(1);
            $generator->setDash(0);
            $generator->setLength(9);
            $generator->setPrefix(self::INVITATION_COUPON_PREFIX);
            $generator->setSuffix('');

            //set the generation settings
            $rule->setCouponCodeGenerator($generator);
            $rule->setCouponType(
                \Magento\SalesRule\Model\Rule::COUPON_TYPE_AUTO
            );

            //generate the coupon
            $coupon = $rule->acquireCoupon();
            $couponCode = $coupon->getCode();

            //save the type of coupon
            /** @var \Magento\SalesRule\Model\Coupon $couponModel */
            $couponModel = $this->couponFactory->create()
                ->loadByCode($couponCode);
            $couponModel->setType(
                \Magento\SalesRule\Model\Rule::COUPON_TYPE_NO_COUPON
            )->setGeneratedByInvitationReferral(1);

            if ($expireDate) {
                $couponModel->setExpirationDate($expireDate);
            } elseif ($rule->getToDate()) {
                $couponModel->setExpirationDate($rule->getToDate());
            }

            $this->coupon->save($couponModel);

            return $couponCode;
        }

        return false;
    }

    public function getInvitationByDbField($field,$value,$return_first = true){
        $collection = $this->invitationCollectionFactory->create();
        $collection->addFieldToSelect('*');
        $collection->addFieldToFilter($field,$value);

        if ($collection->getSize() > 0 && $return_first){
            $invitation = $collection->getFirstItem();
            return $invitation;
        }

        if  ($collection->getSize() > 0){
            return $collection;
        }

        return false;
    }

    public function getInvitationByCouponCode($couponCode){
        $collection = $this->invitationCollectionFactory->create();
        $collection->addFieldToSelect('*');
        $collection->addFieldToFilter(
            array(
                $this->getCouponDbFieldNameForWhoGotInvited(),
                $this->getCouponDbFieldNameForWhoSentTheInvite()
            ),
            array(
                array('eq'=>$couponCode),
                array('eq'=>$couponCode)
            )
        );

        if ($collection->getSize() > 0){
            $invitation = $collection->getFirstItem();
            return $invitation;
        }

        return false;
    }

    public function getCustomerById($id){
        return $this->_customerRepositoryInterface->getById($id);
    }

    public function getCustomerByEmail($email){
        return $this->_customerRepositoryInterface->get($email);
    }

    public function whoInvitedCustomerId($id = 0, $isNewsletter = false){
        $query = "SELECT * FROM magento_invitation_track WHERE referral_id = '{$id}'";

        if ($isNewsletter) {
            $query = "SELECT * FROM filoblu_newsletter_invitation_track WHERE referral_id = '{$id}'";
        }

        $row = $this->getConnection()->fetchRow($query);

        if(isset($row['inviter_id']) && $row['inviter_id']){
            return $row['inviter_id'];
        }else {

            return false;
        }
    }

    public function isInvitationalTypeCoupon($coupon_code=null){
        if (!$coupon_code){
            return false;
        }
        $prefix = $result = substr($coupon_code, 0, 4);
        if ($prefix == self::INVITATION_COUPON_PREFIX){
            return true;
        }
        return false;
    }

    public function getCouponDbFieldNameForWhoSentTheInvite(){
        return self::COUPON_FIELD_NAME_FOR_WHO_SEND_THE_INVITE;
    }

    public function getCouponDbFieldNameForWhoGotInvited(){
        return self::COUPON_FIELD_NAME_FOR_WHO_GET_INVITED;
    }

    public function getSysConfigValue($path,$scope_type=null,$scope_value=null){
        if($scope_type && $scope_value){
            $result = $this->scopeConfig->getValue($path,$scope_type,$scope_value);
        } else {
            $result = $this->scopeConfig->getValue($path);
        }
        return $result;
    }

}
